#include <msp430.h>
#include <stdint.h>
#include <string.h>
#include "dr_dc_motor.h"
#include "clock_setting.h"

#define DRIVE_ENABLE_PORT_DIR   P1DIR |= BIT0  //趨DRV8833оƬʹܶ˿Ϊ
#define MOTOR_DRIVE_ENABLE      P1OUT |= BIT0  //DRV8833оƬʹ
#define MOTOR_DRIVE_DISABLE     P1OUT &= ~BIT0 //DRV8833оƬض

#define PWM_POS_CCR TA0CCR1   //ӦCCRĴ
#define PWM_POS_CTL TA0CCTL1  //ӦCCTLĴ
#define PWM_NEG_CCR TA0CCR2   //ӦCCRĴ
#define PWM_NEG_CTL TA0CCTL2  //ӦCCTLĴ

uint32_t refClock = 0; //תòοʱӣĿǰ趨0.1msһ(PWMزƵͬ)

#define RPM_AVERAGE_COUNT 128 //תٲʱԶٴƽ

struct RPM_MEAS_STRUCT //תٲ״̬õĽṹ
{
  int valid; //ʼΪ-1Ϊ1ʱЧ
  int forward; //Ƿת(Ŀǰʵѹȷ)
  uint32_t lastClkCnt; //һЧֵʱ
  uint32_t curClkCnt; //һЧֵʱ(ڶԶжϹ)

  int intePtr; //ɵһݼ¼λ
  int32_t intervals[RPM_AVERAGE_COUNT]; //ݼ¼
  int32_t inteSum; //ݼ¼֮
} rpm_meas;

void initDCMotor()
{
  //IO˿
  P1SEL |= BIT6 + BIT7; //P1.61.7ΪPWM
  P1DIR |= BIT6 + BIT7; //P1.61.7
  MOTOR_DRIVE_DISABLE; //ʹΪֹͣǰ
  DRIVE_ENABLE_PORT_DIR; //趨ʹܶ˿Ϊ
  //ʱ
  TA0CTL = TASSEL__SMCLK + MC__UP;
  TA0CCR0 = PWM_COUNT; //399
  TA0CCTL0 = CCIE; //100usж
  PWM_POS_CCR = 0;
  PWM_POS_CTL = OUTMOD_5; //reset͵ƽ
  PWM_NEG_CCR = 0;
  PWM_NEG_CTL = OUTMOD_5; //reset͵ƽ
  //תٲ
  P2IE |= BIT0; //P2.0ж
  memset(&rpm_meas, 0, sizeof(rpm_meas));
  rpm_meas.valid = -1;
  rpm_meas.forward = 1;
}

#pragma vector=TIMER0_A0_VECTOR
__interrupt void softTimerInterrupt()
{
  refClock++; //ȫֲοʱ
}

void PWM_SetOutput(int val) // -399 ~ 399
{
  if(val == 0) //ʱֱӹض
  {
      MOTOR_DRIVE_DISABLE;
  }
  else if(val > 0) //תߵƽPWM
  {
    if(val > PWM_COUNT)
      val = PWM_COUNT;

    MOTOR_DRIVE_ENABLE;
    PWM_POS_CCR = val;
    PWM_POS_CTL = OUTMOD_1; //setߵƽ
    PWM_NEG_CCR = val;
    PWM_NEG_CTL = OUTMOD_3; //set/resetvalԽ󣬵͵ƽʱԽЧѹԽ
    rpm_meas.forward = 1;
  }
  else //תߵƽPWM
  {
    if(val < -PWM_COUNT)
      val = -PWM_COUNT;

    MOTOR_DRIVE_ENABLE;
    PWM_POS_CCR = val;
    PWM_POS_CTL = OUTMOD_3; //set/resetvalԽ󣬵͵ƽʱԽЧѹԽ
    PWM_NEG_CCR = val;
    PWM_NEG_CTL = OUTMOD_1; //setߵƽ
    rpm_meas.forward = 0;
  }
}

#pragma vector=PORT2_VECTOR
__interrupt void p2Interrupt() //תٲж
{
  P2IES ^= BIT0; //תط
  if(refClock - rpm_meas.curClkCnt >= 2) //ֻм0.2msΪЧ(0.2msٶӦת9000rpmǲܵ)
  {
    rpm_meas.lastClkCnt = rpm_meas.curClkCnt;
    rpm_meas.curClkCnt = refClock; //¼ǰʱ
    if(rpm_meas.valid < 1) //ǷѾ¼㹻ڼ
      rpm_meas.valid++;
    else
    {
      rpm_meas.inteSum -= rpm_meas.intervals[rpm_meas.intePtr];
      rpm_meas.intervals[rpm_meas.intePtr] = refClock - rpm_meas.lastClkCnt;
      rpm_meas.inteSum += rpm_meas.intervals[rpm_meas.intePtr];
      rpm_meas.intePtr++;
      if(rpm_meas.intePtr >= RPM_AVERAGE_COUNT)
        rpm_meas.intePtr = 0;
    }
  }
  P2IFG = 0; //жϱ־
}

int getRPM() //ݵǰתٲתֵλΪRPM
{
  if(rpm_meas.valid < 1) //δ¼㹻
    return 0;

  if(refClock - rpm_meas.curClkCnt > 2500L) //250msӦٶȵ60rpmΪЧ
  {
    rpm_meas.valid = -1;
    return 0;
  }

  int32_t val = rpm_meas.inteSum; //ֵָı䣬ȼ¼ȡֵ
  //ת٣λrpm
  //60s * ÿʱӼ(10k) / (ƽֵ / ÿתһȦ)
  val = 60L * 1000L * 10L * (RPM_AVERAGE_COUNT / 2) / 4L / val;
  if(rpm_meas.forward)
    return val;
  else
    return -val;
}
